/*
 * Copyright (C) 1995 M. Hauber, Ch. Schneider, G. Caronni
 * See COPYING for more details
 */
#include "config.h"
#include "skip_defs.h"
#include "dynamic.h"
#include "memblk.h"
#include "crypt.h"
#include "sign.h"
#include "signmd5.h"
#include "skipcache.h"
#include "random.h"

#ifdef __GNUC__
#ident "$Id: random.c,v 1.7 1996/04/25 15:05:30 cschneid Exp $"
#else
static char rcsid[] = "$Id: random.c,v 1.7 1996/04/25 15:05:30 cschneid Exp $";
#endif

/**************************************************************************
 * RC4 Implementation from Usenet source                                  *
 **************************************************************************/
 
typedef struct rc4_key
{
  unsigned char state[256];
  unsigned char x, y;
} rc4_key;

static SEMTYPE semaphore;
static rc4_key state;
 
static void rc4_setkey(unsigned char *keydata, int len, rc4_key *key)
{
  unsigned char i2, is, i2s, *state, *k;
  int i, i1;
 
  state = key->state;
  for (i = 0; i < 256; i++)
    *state++ = i;
 
  state = key->state;
  for (i = i1 = i2 = 0, k = keydata; i < 256; i++)
  {
    is = state[i];
    i2 += *k++ + is;
    i2s = state[i2];
    state[i] = i2s;
    state[i2] = is;
    
    if (++i1 >= len)
    {
      k = keydata;
      i1 = 0;
    }
  }
 
  key->x = key->y = 0;
}

static void rc4_crypt(unsigned char *src, unsigned char *dst, int len, rc4_key *
key)
{
  int i;
  unsigned char x, y, xs, ys, *state;
 
  x = key->x;
  y = key->y;
  state = key->state;
 
  for (i = 0; i < len; i++)
  {
    xs = state[++x];
    ys = state[y += xs];
    state[x] = ys;
    state[y] = xs;
    *dst++ = *src++ ^ state[(unsigned char)(xs + ys)];
  }
 
  key->x = x;
  key->y = y;
}

int random_hash(u_char *buf, int buflen, u_char *val, int vallen)
{
  int digestlen = sign_icvlen(SIGNALG_MD5);
  int len;

  for (len = 0; buflen > 0; len += digestlen, buflen -= digestlen)
  {
    struct memblk m;
    struct memseg ms;
    u_char hash[64];

    m.len = len;
    m.offset = 0;
    m.ms = &ms;
    ms.len = len;
    ms.ptr = buf;
    sign_sign(SIGNALG_MD5, val, vallen, &m, hash);
    MEMCPY(buf + len, hash, (digestlen > buflen) ? buflen : digestlen);
  }

  return 0;
}

int random_expandKp(u_char *Kp, int Kp_len, u_char *E_Kp, int E_Kp_len, 
		    u_char *A_Kp, int A_Kp_len, u_char cryptalg, u_char macalg)
{
  MD5_CTX md5template, md5ctx;
  char ch;

  /* Calc MD5 context of Kp only once */
  MD5Init(&md5template);
  MD5Update(&md5template, Kp, Kp_len);
  
  /* Calculate E_Kp */
  if (E_Kp_len > sizeof(md5ctx.digest))
  {
    int len = E_Kp_len - sizeof(md5ctx.digest);

    /* Calc high order part of E_Kp */
    MEMCPY(&md5ctx, &md5template, sizeof(md5ctx));
    ch = cryptalg;
    MD5Update(&md5ctx, &ch, 1);
    ch = 2;
    MD5Update(&md5ctx, &ch, 1);
    MD5Final(&md5ctx);
    
    MEMCPY(E_Kp, md5ctx.digest + sizeof(md5ctx.digest) - len, len);
    E_Kp += len;
    E_Kp_len = sizeof(md5ctx.digest);
  }
  
  /* Calc low order part of E_Kp */
  MEMCPY(&md5ctx, &md5template, sizeof(md5ctx));
  ch = cryptalg;
  MD5Update(&md5ctx, &ch, 1);
  ch = 0;
  MD5Update(&md5ctx, &ch, 1);
  MD5Final(&md5ctx);
  
  MEMCPY(E_Kp, md5ctx.digest + sizeof(md5ctx.digest) - E_Kp_len, E_Kp_len);
  
  /* Calculate A_Kp */
  if (A_Kp_len > sizeof(md5ctx.digest))
  {
    int len = A_Kp_len - sizeof(md5ctx.digest);

    /* Calc high order part of E_Kp */
    MEMCPY(&md5ctx, &md5template, sizeof(md5ctx));
    ch = macalg;
    MD5Update(&md5ctx, &ch, 1);
    ch = 3;
    MD5Update(&md5ctx, &ch, 1);
    MD5Final(&md5ctx);
    
    MEMCPY(A_Kp, md5ctx.digest + sizeof(md5ctx.digest) - len, len);
    A_Kp += len;
    A_Kp_len = sizeof(md5ctx.digest);
  }
  
  /* Calc low order part of E_Kp */
  MEMCPY(&md5ctx, &md5template, sizeof(md5ctx));
  ch = macalg;
  MD5Update(&md5ctx, &ch, 1);
  ch = 1;
  MD5Update(&md5ctx, &ch, 1);
  MD5Final(&md5ctx);
  
  MEMCPY(A_Kp, md5ctx.digest + sizeof(md5ctx.digest) - A_Kp_len, A_Kp_len);

  return 0;
}

/*
 * Generate a random packet key Kp for given skipcache entry
 */
int random_randomKp(struct skipcache *c)
{
  struct memblk m;
  struct memseg ms;
  u_char MI[MAXBLOCKLEN];
  static u_char es[1024];  /* Encrypt state */

  if ((c->flags) & SKIP_VALIDKEY)
  {
    int Kp_Kijn_keylen = c->enskip_Kp_Kijn.key.len;
    int Kij_keylen = crypt_keylen(c->enskip_Kij_alg);

    MEMZERO(MI, sizeof(MI));

    c->enskip_Kp_ttl = c->enskip_Kp_maxttl;
    c->enskip_Kp_bytes = 0;

    /* Randomize MI for block ciphers, set to 0 for stream ciphers */
    if (crypt_blocklen(c->enskip_Crypt_alg) > 1)
      random_random(c->data + c->enskip_MI.offset, c->enskip_MI.len);
    else
      MEMZERO(c->data + c->enskip_MI.offset, c->enskip_MI.len);

    random_random(c->data + c->enskip_Kp_Kijn.key.offset, Kp_Kijn_keylen);
    random_expandKp(c->data + c->enskip_Kp_Kijn.key.offset, Kp_Kijn_keylen,
		    c->data + c->enskip_E_Kp.offset, c->enskip_E_Kp.len,
		    c->data + c->enskip_A_Kp.offset, c->enskip_A_Kp.len,
		    c->enskip_Crypt_alg, c->enskip_MAC_alg);

    m.ms = &ms;
    m.len = Kp_Kijn_keylen;
    m.offset = 0;
    ms.ptr = c->data + c->enskip_Kp_Kijn.key.offset;
    ms.len = Kp_Kijn_keylen;
    SEMLOCK(semaphore);                /* Protect encrypt state */
    crypt_encrypt(c->enskip_Kij_alg, es, 
		  c->data + c->Kijn[skip_n % 3].key.offset + 
		  c->Kijn[skip_n % 3].key.len - Kij_keylen,
		  MI, MI, &m, &m);
    SEMUNLOCK(semaphore);
    c->enskip_Kp_Kijn.n = skip_n;
  }

  return 0;
}

int random_init(void)
{
  SEEDTYPE seed;
  u_char rc4seed[256];

  SEMALLOC(semaphore);
  SEMINIT(semaphore);
  SEEDINIT(&seed);
  random_hash(rc4seed, sizeof(rc4seed), (u_char *)&seed, sizeof(seed));
  MEMZERO(&seed, sizeof(seed));
  rc4_setkey(rc4seed, sizeof(rc4seed), &state);
  MEMZERO(rc4seed, sizeof(rc4seed));
  return 0;
}

int random_exit(void)
{
  SEMFREE(semaphore);
  return 0;
}

int random_random(u_char *buf, int len)
{
  SEMLOCK(semaphore);
  rc4_crypt(buf, buf, len, &state);
  SEMUNLOCK(semaphore);
  return 0;
}
