/*
   Copyright (C) 1996 Robert Muchsel

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   Please address all correspondence concerning the software to
   muchsel@acm.org.
*/

#include "config.h"
#include <linux/config.h>
#define __NO_VERSION__
#include <linux/module.h>
#include <netinet/ip.h>
#include <netinet/udp.h>
#include <netinet/ip_icmp.h>
#include <linux/ip.h>
#include <linux/in.h>
#include <linux/skbuff.h>
#include <linux/netdevice.h>
#include <linux/firewall.h>
#include <linux/string.h>
#include <linux/ipsec.h>
#include <net/route.h>
#include <net/icmp.h>
#include "skip_defs.h"
#include "dynamic.h"
#include "memblk.h"
#include "skipcache.h"
#include "ipsp.h"
#include "skip.h"
#include "ipsum.h"
#include "interface.h"

/* #define DEBUG_LINUX_MIN_OUT	*/
/* #define DEBUG_LINUX_OUTPUT	*/
/* #define DEBUG_LINUX_MIN_IN	*/
/* #define DEBUG_LINUX_INPUT	*/
/* #define DEBUG_LINUX_SK	*/
/* #define DEBUG_LINUX_GETBUF	*/
#define DEBUG_LINUX_USERS	/* */
/* #define DEBUG_LINUX_PACKETS	*/
/* #define DEBUG_LINUX_CHK_CRYPT*/
#define DEBUG_LINUX_PROT	/* */
#define DEBUG_LINUX_ICMP	/* */
/* #define DEBUG_LINUX_MEM	*/

static struct sk_buff *expand_skb(struct sk_buff *, int);
static inline void skb2memblk(struct sk_buff *, struct memblk *, struct memseg *);
static inline void interface_ship_out(struct sk_buff *);

static int maxheadergrowth = 0;

#if defined(DEBUG_LINUX_PACKETS) || defined(DEBUG_LINUX_CHK_CRYPT)
static void packet_dump(char *msg, u_char *c, int len)
{
  printk(msg);
  while (len--)
    printk(" %02x", *c++);
  printk("\n");
}
#endif


/* This function copies an skb, prepending "space" bytes. 
   Mostly derived from the Linux 2.0.25 skb_copy function. */

static struct sk_buff *expand_skb(struct sk_buff *skb, int space)
{
  struct sk_buff *n;
  unsigned long offset;

  /* Allocate the copy buffer */

  IS_SKB(skb);

  n = alloc_skb(skb->truesize - sizeof(struct sk_buff) + space, GFP_ATOMIC);
  if (n == NULL)
    return NULL;
#ifdef DEBUG_LINUX_MEM
  printk("alloc: skb %i\n", n->truesize);
#endif

  /* Shift between the two data areas in bytes */
  offset = n->head - skb->head + space;

  /* Set the data pointer */
  skb_reserve(n, skb->data - skb->head + space);

  /* Set the tail pointer and length */
  skb_put(n, skb->len);

  /* Copy the bytes */
  memcpy(n->head + space, skb->head, skb->end - skb->head);
  n->link3   = NULL;
  n->list    = NULL;
  n->sk      = NULL;
  n->when    = skb->when;
  n->dev     = skb->dev;
  n->h.raw   = skb->h.raw + offset;
  n->mac.raw = skb->mac.raw + offset;
  n->ip_hdr  = (struct iphdr *)(((char *)skb->ip_hdr) + offset);
  n->saddr   = skb->saddr;
  n->daddr   = skb->daddr;
  n->raddr   = skb->raddr;
  n->seq     = skb->seq;
  n->end_seq = skb->end_seq;
  n->ack_seq = skb->ack_seq;
  n->acked   = skb->acked;
  memcpy(n->proto_priv, skb->proto_priv, sizeof(skb->proto_priv));
  n->used    = skb->used;
  n->free    = 1;
  n->arp     = skb->arp;
  n->tries   = 0;
  n->lock    = 0;
  n->users   = 0;
  n->pkt_type= skb->pkt_type;
  n->stamp   = skb->stamp;

  IS_SKB(n);
        
  return n;
}


/* This function translates an skb to the memblk structure, which is
   the internal ENskip representation of "memory" (to be compatible
   with BSD and other UNIXes, where IP packets are stored in multiple
   chained mbuf's (or mblk's). Under Linux, we don't need this, the
   drawback is however that prepending a header requires to copy the
   whole data area. Free tailroom seems to be unused... */

static inline void skb2memblk(struct sk_buff *skb, 
                              struct memblk *mb, struct memseg *ms)
{
  /* no "dynamic" part (== M_EXT of mbuf) */
  mb->dynamic    = NULL;
  mb->dynamiclen = 0;

  /* first "segment descriptor" (the one and only we actually use) */
  mb->ms     = ms;
  /* offset == offset of IP header in data area */
  mb->offset = (u_char *) skb->ip_hdr - skb->head;
  /* len == length of IP packet */
  mb->len    = ntohs(skb->ip_hdr->tot_len);

  /* ptr == pointer to the data area in first segment descriptor */
  ms->ptr = skb->head;
  /* length of data block in first segment descriptor */
  ms->len = mb->len + mb->offset;

  /* first free memseg, ESP uses this internally */
  mb->freems = ms+1;
}


/* This function intercepts incoming IP packets, both local and non-local.
   We return FW_BLOCK (which causes the kernel to discard the packet)
   if we queue the packet or want to dispose it. We return FW_SKIP if
   the packet was not enskipped or wasn't changed. */

int input_packet(struct firewall_ops *this, int pf, struct device *dev,
		 void *phdr, void *arg, struct sk_buff **pskb)
{
  struct iphdr *ipp = (*pskb)->ip_hdr;
  int tot_len = ntohs(ipp->tot_len);
  struct sk_buff *newm;
  int result;
  struct memblk oldmb, newmb;
  struct memseg oldms[STATIC_MEMSEGS], newms[2];

  /* leave short packets to the firewall */
  if (tot_len < ipp->ihl * 4)
    return FW_SKIP;

  /* hack packet to decrease maximum MTU */
  if ((ipp->protocol == IPPROTO_ICMP) && 
      (tot_len >= ipp->ihl * 4 + sizeof(struct icmphdr))) {
    struct icmphdr *icmp = (struct icmphdr *) (((u_char *) ipp) + ipp->ihl * 4);

#ifdef DEBUG_LINUX_PROT
    if (icmp->type == ICMP_DEST_UNREACH && 
        icmp->code == ICMP_PROT_UNREACH) 
      printk("enskip: icmp: protocol unreachable\n");
#endif

    if (icmp->type == ICMP_DEST_UNREACH && 
        icmp->code == ICMP_FRAG_NEEDED) {
      __u32 checksum = icmp->checksum;
      __u16 mtu = ntohs(icmp->un.echo.sequence);
      __u16 diff = maxheadergrowth;

#ifdef DEBUG_LINUX_ICMP
      struct iphdr *iph = (struct iphdr *) ((u_char *) icmp + 8);
      printk("enskip: frag: mtu=%i-%i, len was %i\n", mtu, diff,
             ntohs(iph->tot_len));
#endif

      /* we calculate the checksum in network byte order */
      mtu -= diff;
      icmp->un.echo.sequence = htons(mtu);

      checksum += htons(diff);
      checksum += checksum >> 16;
      icmp->checksum = checksum;

      ip_send_check(ipp);
    }

  }

  if ((*pskb)->proto_priv[15] & RCV_SEC) {
#ifdef DEBUG_LINUX_INPUT
    printk("in: old packet (p=%i)\n", ipp->protocol); 
#endif
    return FW_SKIP;
  }

  /* never change transparent UDP ports needed for discovery */
  if ((ipp->protocol == IPPROTO_UDP) && 
      (tot_len >= ipp->ihl * 4 + sizeof(struct udphdr))) {
    struct udphdr *udp = (struct udphdr *) (((u_char *) ipp) + ipp->ihl * 4);

    if ((udp->source == htons(SKIP_UDP_RECV_PORT)) || 
        (udp->source == htons(SKIP_UDP_SEND_PORT)) ||
        (udp->dest   == htons(SKIP_UDP_RECV_PORT)) || 
        (udp->dest   == htons(SKIP_UDP_SEND_PORT)))
      return FW_SKIP;
  }
	
  /* don't unlink in the middle of a turnaround */
  MOD_INC_USE_COUNT;
	
#ifdef DEBUG_LINUX_INPUT
  printk("%p: in: p=%i l=%hu s=%08x d=%08x\n", 
         *pskb, ipp->protocol, tot_len, ipp->saddr, ipp->daddr);
#endif

  /* If the number of users is 0 or 1, we may write to the original buffer.
     Else we prepend ample space to avoid being called again (in getbuf). */
  if ((*pskb)->users > 1)
    newm = expand_skb(*pskb, 1024);     /* no? avoid being called again */
  else
    newm = skb_clone(*pskb, GFP_ATOMIC);/* overwrite current data area */
  if (newm == NULL) {
    /* out of memory... */
    MOD_DEC_USE_COUNT;
    return FW_BLOCK;
  }

  MEMZERO(&newmb, sizeof(newmb));
  newmb.ms = newms;
  MEMZERO(&newmb, sizeof(newmb));
  oldmb.ms = oldms;
  skb2memblk(*pskb, &oldmb, oldms);
  skb2memblk(newm, &newmb, newms);

  result = skip_process(SKIP_INPUT, NULL, NULL, (void *) &newm, &oldmb, &newmb);
#ifdef DEBUG_LINUX_INPUT
  printk("in: skip_process, result=");
#endif

  if (result == SKIP_PROCESSED) {
    /* nothing happened */
#ifdef DEBUG_LINUX_INPUT
    printk("PASSED\n");
#endif
#ifdef DEBUG_LINUX_MEM
    printk("free: skb %i\n", newm->truesize);
#endif
    kfree_skb(newm, FREE_WRITE);
    MOD_DEC_USE_COUNT;
    return FW_SKIP;
  }
  else if (result > SKIP_PROCESSED) {
    /* detunneled/decrypted/authenticated */

    /* fix up the skb pointers */
    newm->data      = BLKSTART(&newmb);
    newm->ip_hdr    = (struct iphdr *) newm->data;
    newm->len       = ntohs(newm->ip_hdr->tot_len);
    newm->tail      = newm->data + newm->len;
    newm->protocol  = htons(ETH_P_IP);
    newm->ip_summed = 0;
    newm->h.iph     = newm->ip_hdr;

#ifdef DEBUG_LINUX_INPUT
    printk("PROCESSED\n");
#endif
#if defined(DEBUG_LINUX_MIN_IN) || defined(DEBUG_LINUX_INPUT)
    printk("in: p=%i>%i l=%i>%i s=%08x d=%08x\n", 
           (*pskb)->ip_hdr->protocol, newm->ip_hdr->protocol, 
           ntohs((*pskb)->ip_hdr->tot_len), ntohs(newm->ip_hdr->tot_len), 
           newm->ip_hdr->saddr, newm->ip_hdr->daddr);
#endif
#ifdef DEBUG_LINUX_PACKETS
    packet_dump("in: new=", newm->data, 10);
#endif

    /* and mark the skb as "authenticated"/"decrypted" */
    newm->proto_priv[15] = ((result & SKIP_P_AUTH)    ? RCV_AUTH   : 0)
                         | ((result & SKIP_P_DECRYPT) ? RCV_CRYPT  : 0)
                         | ((result & SKIP_P_TUNNEL)  ? RCV_TUNNEL : 0)
                         | RCV_SEC;

    /* and feed the packet back into the input queue (must not switch
       skbs here, because we need the defragmentor) */
#ifdef DEBUG_LINUX_MEM
    printk("free: in netif_rx: %i\n", newm->truesize);
#endif
    netif_rx(newm); /* frees skb for us */
  } 
  else if (result == SKIP_QUEUED) {
    /* the skb was queued */
#ifdef DEBUG_LINUX_INPUT
    printk("QUEUED\n");
#endif
  } else {
    /* bad packet, policy violation, unsupported protocol, etc. */
    printk("in: DISCARD\n");
#ifdef DEBUG_LINUX_MEM
    printk("free: skb %i\n", newm->truesize);
#endif
    kfree_skb(newm, FREE_WRITE);
  }
  
  MOD_DEC_USE_COUNT;
  return FW_BLOCK;
}


/* This is the output interceptor. Basically this function works like
   the input interceptor, however the feedback function is not so
   "nice" (we use ip_forward). There's also a new return code, 
   FW_QUEUE, which works like FW_BLOCK but does not cause a "permission
   denied" message on local packets (kernel function ip_build_xmit). */

int output_packet(struct firewall_ops *this, int pf, struct device *dev,
		  void *phdr, void *arg, struct sk_buff **pskb)
{
  struct iphdr *ipp = (*pskb)->ip_hdr;
  int tot_len = ntohs(ipp->tot_len);
#ifdef NOTUSED_CAUSE_BROKEN
  __u32 daddr = ipp->daddr;
#endif
  struct sk_buff *newm;
  int result;
  struct memblk oldmb, newmb;
  struct memseg oldms[STATIC_MEMSEGS], newms[2];
#ifdef DEBUG_LINUX_CHK_CRYPT
  struct sk_buff *newm2;
  struct memblk newmb2;
  struct memseg newms2[2];
#endif

  /* Recursion happens if the datagram has been fragmented:
     ip_queue_xmit -> skip -> ip_fragment -> ip_queue_xmit -> SKIP */
  if (ipp->protocol == IPPROTO_SKIP)
    return FW_ACCEPT;

  /* never change trasparent UDP packets */ 
  if ((ipp->protocol == IPPROTO_UDP) &&
      (tot_len >= ipp->ihl * 4 + sizeof(struct udphdr))) {
    struct udphdr *udp = (struct udphdr *) (((u_char *) ipp) + ipp->ihl * 4);

    if ((udp->source == htons(SKIP_UDP_RECV_PORT)) || 
        (udp->source == htons(SKIP_UDP_SEND_PORT)) ||
        (udp->dest   == htons(SKIP_UDP_RECV_PORT)) || 
        (udp->dest   == htons(SKIP_UDP_SEND_PORT)))
      return FW_SKIP;
  }

  if ((*pskb)->sk != NULL) {
#ifdef DEBUG_LINUX_SK
    printk("out: socket %p\n", (*pskb)->sk);
#endif

    if ((*pskb)->sk->authentication == IPSEC_LEVEL_NONE &&
        (*pskb)->sk->encryption     == IPSEC_LEVEL_NONE)
      return FW_SKIP;
  }


#ifdef DEBUG_LINUX_OUTPUT
  printk("%p: out[%i]: p=%i l=%hu s=%08x d=%08x\n", 
         *pskb, (int) arg, ipp->protocol, tot_len, ipp->saddr, ipp->daddr);
#endif

  /* don't unlink in the middle of a turnaround */
  MOD_INC_USE_COUNT;

#ifdef DEBUG_LINUX_PACKETS
  packet_dump("out: old=", (u_char *) ipp, 10);
#endif

#ifdef DEBUG_LINUX_USERS
  if ((*pskb)->users > 0)
    printk("out: users=%i\n", (*pskb)->users); 
#endif

  /* If there's no user on the skb, we may change the data area.
     Else we copy the data, prepending enough space so that we will not
     have to copy again (in the getbuf function) */
  if ((*pskb)->users > 1) 
    newm = expand_skb(*pskb, maxheadergrowth);
  else 
    /* overwrite the data area */
    newm = skb_clone(*pskb, GFP_ATOMIC);
  if (newm == NULL) {
    /* out of memory... */
    MOD_DEC_USE_COUNT;
    return FW_BLOCK;
  }

  MEMZERO(&newmb, sizeof(newmb));
  newmb.ms = newms;
  MEMZERO(&oldmb, sizeof(oldmb));
  oldmb.ms = oldms;
  skb2memblk(*pskb, &oldmb, oldms);
  skb2memblk(newm, &newmb, newms);

  result = skip_process(SKIP_OUTPUT, NULL, NULL, (void *) &newm, &oldmb, &newmb);

#ifdef DEBUG_LINUX_OUTPUT
  printk("output %p: skip_process result=", newm);
#endif

  if (result == SKIP_PROCESSED) {
    /* nothing happened - if not enskipped, we just say "OK" */ 

#ifdef DEBUG_LINUX_OUTPUT
    printk("SKIP\n");
#endif
#ifdef DEBUG_LINUX_MEM
    printk("free: skb %i\n", newm->truesize);
#endif
    kfree_skb(newm, FREE_WRITE);

    MOD_DEC_USE_COUNT;

    /* check user level policy */
    if ((*pskb)->sk != NULL &&
        ((*pskb)->sk->authentication >= IPSEC_LEVEL_USE ||
         (*pskb)->sk->encryption     >= IPSEC_LEVEL_USE))
      return FW_BLOCK;

    return FW_SKIP;
  } 
  else if (result > SKIP_PROCESSED) {
    /* tunneled/encrypted/authenticated */

    /* check user level policy */
    if ((*pskb)->sk &&
        ((!(result & SKIP_P_AUTH) && 
          (*pskb)->sk->authentication >= IPSEC_LEVEL_USE) ||
         (!(result & SKIP_P_ENCRYPT) &&
          (*pskb)->sk->encryption >= IPSEC_LEVEL_USE))) {
#ifdef DEBUG_LINUX_MEM
      printk("free: skb %i\n", newm->truesize);
#endif
      kfree_skb(newm, FREE_WRITE);
      return FW_BLOCK;
    }

    /* fix up skb */
    newm->data      = BLKSTART(&newmb);
    newm->ip_hdr    = (struct iphdr *) newm->data;
    newm->len       = ntohs(newm->ip_hdr->tot_len);
    newm->tail      = newm->data + newm->len;
    newm->protocol  = htons(ETH_P_IP);
    newm->ip_summed = 0;
    newm->h.iph     = newm->ip_hdr;

#ifdef DEBUG_LINUX_OUTPUT
    printk("PROCESSED\n");
#endif
#if defined(DEBUG_LINUX_MIN_OUT) || defined(DEBUG_LINUX_OUTPUT)
    printk("out[%i]: p=%i>%i l=%i>%i s=%08x d=%08x\n", 
           (int) arg, (*pskb)->ip_hdr->protocol, newm->ip_hdr->protocol,
           ntohs((*pskb)->ip_hdr->tot_len), ntohs(newm->ip_hdr->tot_len), 
           newm->ip_hdr->saddr, newm->ip_hdr->daddr);
#endif
#ifdef DEBUG_LINUX_CHK_CRYPT
    newm2 = expand_skb(newm, maxheadergrowth);
    MEMZERO(&newmb2, sizeof(newmb2));
    newmb2.ms = newms2;
    skb2memblk(newm2, &newmb2, newms2);
    skip_process(SKIP_INPUT, NULL, NULL, (void *) &newm2, &newmb, &newmb2);
    newm2->data      = BLKSTART(&newmb2);
    newm2->ip_hdr    = (struct iphdr *) newm2->data;
    newm2->len       = ntohs(newm2->ip_hdr->tot_len);
    newm2->tail      = newm2->data + newm2->len;
    newm2->protocol  = htons(ETH_P_IP);
    newm2->ip_summed = 0;
    newm2->h.iph     = newm2->ip_hdr;
    if (newm2->ip_hdr->protocol != (*pskb)->ip_hdr->protocol) {
      printk("tst[%i]: p=%i>%i l=%i>%i\n", 
             (int) arg, newm->ip_hdr->protocol, newm2->ip_hdr->protocol,
             ntohs(newm->ip_hdr->tot_len), ntohs(newm2->ip_hdr->tot_len));
      printk("!!! encryption error\n");
      packet_dump("orig:", (*pskb)->ip_hdr, 40);
      packet_dump("decr:", newm2->ip_hdr, 40);
    }
    kfree_skb(newm2, FREE_WRITE);
#endif  
#ifdef NOTUSED_CAUSE_BROKEN
    /* XXX switch sockets; fix link */
    if (newm->ip_hdr->daddr == daddr) {
      /* switch skbs */
      kfree_skb(*pskb, FREE_WRITE);
      /* and hope nobody will notice */ 
      *pskb = newm;
      MOD_DEC_USE_COUNT;
      return FW_SKIP;
    }
    else
#endif

    interface_ship_out(newm);
  }
  else if (result == SKIP_QUEUED) {
    /* queued, will be fed back to us */
#ifdef DEBUG_LINUX_OUTPUT
    printk("QUEUED\n");
#endif
  } 
  else {
    /* bad packet/policy/etc. */
    printk("out: DISCARD\n");
#ifdef DEBUG_LINUX_MEM
    printk("free: skb %i\n", newm->truesize);
#endif
    kfree_skb(newm, FREE_WRITE);
    MOD_DEC_USE_COUNT;
    return FW_BLOCK;
  }

  MOD_DEC_USE_COUNT;
  return FW_QUEUE;
}


/* Forward a packet. Basically this is the same as output_packet,
   however we call the firewall output routine first. */

int forward_packet(struct firewall_ops *this, int pf, struct device *dev,
		  void *phdr, void *arg, struct sk_buff **pskb)
{
  if (pskb == 0) {
    /* hack to return true device MTU; arg is pointer to MTU */

    if ((dev->flags & IFF_SKIP) && arg)
      *((unsigned short *) arg) += maxheadergrowth;
    return FW_ACCEPT;
  }
  else {
    struct iphdr *iph = (*pskb)->h.iph;
    unsigned long checksum = iph->check;
    int fw_res;

    /* give firewalls a chance to get the packet unencrypted */
    if ((fw_res = call_fw_firewall(PF_INET, (*pskb)->dev, iph, NULL, pskb)) < FW_ACCEPT) {
      if (fw_res ==  FW_REJECT)
        icmp_send(*pskb, ICMP_DEST_UNREACH, ICMP_HOST_UNREACH, 0, (*pskb)->dev);
      return fw_res;
    }

    /* decrease the ttl - in network byte order */
    iph->ttl--;
    checksum += htons(0x0100);
    checksum += checksum >> 16;
    iph->check = checksum;

    return output_packet(this, pf, dev, phdr, arg, pskb);
  }
}


/* and the null op */

int packet_nop(struct firewall_ops *this, int pf, struct device *dev,
	       void *phdr, void *arg, struct sk_buff **pskb)
{
  return FW_SKIP;
}

/* and one for keeping away standard firewalls -- on forward, a standard
   forward firewall call would be made. This call would also be made
   with local packets AND it would get the encrypted packet. Would
   sf care ?? XXX */ 

int packet_accept(struct firewall_ops *this, int pf, struct device *dev,
	       void *phdr, void *arg, struct sk_buff **pskb)
{
  if (((struct iphdr *) phdr)->protocol == IPPROTO_SKIP)
    return FW_ACCEPT;
  else
    return FW_SKIP;
}


/* Put skb back to kernel output processing. Caution! Causes
   the output interceptor to be called recursively. Since it
   is done via the timer, there should be no way to crash the
   stack!? */

static inline void interface_ship_out(struct sk_buff *skb)
{
#ifdef DEBUG_LINUX_OUTPUT
  struct iphdr *ipp = skb->ip_hdr;
#endif
  int result;

#ifdef DEBUG_LINUX_OUTPUT
  printk("ship_out %p: p=%i s=%08x d=%08x\n", skb, ipp->protocol, 
         ipp->saddr, ipp->daddr);
#endif

  IS_SKB(skb);

  MOD_INC_USE_COUNT;

  result = ip_forward(skb, skb->dev, IPFWD_NOTTLDEC, skb->h.iph->daddr);

#ifdef DEBUG_LINUX_OUTPUT
  printk("ship_out: transmitted to ip_forward, result=%i\n", result);
#endif
  if (result) {
#ifdef DEBUG_LINUX_MEM
      printk("free: skb %i\n", skb->truesize);
#endif
      kfree_skb(skb, FREE_WRITE);
  }
#ifdef DEBUG_LINUX_MEM
  else
    printk("free: in ip_forward\n");
#endif

  MOD_DEC_USE_COUNT;
}


/* feed back queued packet into input stream */
int interface_feed_out(struct sk_buff *skb)
{
  IS_SKB(skb);

  /* fix the packet for ip_forward (because ip_build_xmit might not have) */
  skb->h.iph = skb->ip_hdr;

  /* check it */
  if (output_packet(NULL, PF_IPSEC, NULL, NULL, NULL, &skb) < FW_ACCEPT) {
#ifdef DEBUG_LINUX_MEM
    printk("free: skb %i\n", skb->truesize);
#endif
    kfree_skb(skb, FREE_WRITE);
  }
  else
    /* ...and ship it */
    interface_ship_out(skb);
  return 0;
}


/* feed back queued packet into input stream */
int interface_feed_in(struct sk_buff *skb)
{
  skb->protocol  = htons(ETH_P_IP);
  skb->ip_summed = 0;
#ifdef DEBUG_LINUX_MEM
  printk("free: in netif_rx: %i\n", skb->truesize);
#endif
  netif_rx(skb); /* frees skb for us */
  return 0;
}


/* Prepend "size" bytes to skb -- requires a copy of the skb.
   Note: we don't change the data/tail/len fields of the skb,
   but leave this for the code in the packet filter */

int interface_getbuf(struct memblk *m, void **pskb, int size)
{
  struct sk_buff *skb2;

  /* we might be lucky and have enough room in the skb */
  if (m->offset < size) {
#ifdef DEBUG_LINUX_GETBUF
    printk("expand %p: size=%i / old room=%i\n", *pskb, size, m->offset);
#endif
    /* no. allocate a new one. */
    skb2 = expand_skb(*pskb, size /*- m->offset*/);
    if (skb2 == NULL)
      return -ENOMEM;

#ifdef DEBUG_LINUX_MEM
    printk("free: skb %i\n", ((struct sk_buff *) (*pskb))->truesize);
#endif
    kfree_skb((struct sk_buff *) *pskb, FREE_WRITE);
    (struct sk_buff *) *pskb = skb2;

    m->ms->ptr = skb2->head;
    m->offset  = 0; /* always 0 - think! */
  }
  else
    m->offset -= size;

  m->len     += size;
  m->ms->len += size;

  return 0;
}

/* We are attached to all interfaces */
int interface_attach(void *qptr, u_char *ipaddr)
{
  return 0;
}

int interface_detach(void *qptr, u_char *ipaddr)
{
  return 0;
}


/* Decrease the MTUs of all SKIP interfaces */

int interface_init(void)
{
  struct device *dev;

  maxheadergrowth = ipsp_maxheadergrowth();

  for (dev = dev_base; dev != NULL; dev = dev->next) { 
    if (dev->mtu < 68 + maxheadergrowth)
      printk("enskip: %s: interface mtu of %d is too small\n",
              dev->name, dev->mtu);
    else {
      dev->mtu   -= maxheadergrowth;
      dev->flags |= IFF_SKIP;
    }
  }

  return 0;
}


/* Restore the MTUs */

int interface_exit(void)
{
  struct device *dev;

  for (dev = dev_base; dev != NULL; dev = dev->next) { 
    if (dev->flags & IFF_SKIP) {
      dev->flags &= ~IFF_SKIP;
      dev->mtu   += maxheadergrowth;
    }
  }

  return 0;
}
